@@ -52,8 +52,16 @@
         }                                                               \
 }
 
-#undef  MIN
-#define MIN(a, b)  (((a) < (b)) ? (a) : (b))
+#define CC2(number,t1,t2,v,maxlen)                                      \
+{                                                                       \
+	if ((number != t1) && (number != t2)) {                         \
+		snprintf (v, maxlen,                                    \
+			_("Invalid number of components (%i, "          \
+			"expected %i or %i)."), (int) number,		\
+			(int) t1, (int) t2);  				\
+		break;                                                  \
+	}                                                               \
+}
 
 static struct {
 	ExifTag tag;
@@ -156,7 +164,7 @@ static struct {
 char *
 mnote_olympus_entry_get_value (MnoteOlympusEntry *entry, char *v, unsigned int maxlen)
 {
-	char         buf[32];
+	char         buf[30];
 	ExifLong     vl;
 	ExifShort    vs = 0;
 	ExifRational vr;
@@ -179,7 +187,7 @@ mnote_olympus_entry_get_value (MnoteOlympusEntry *entry, char *v, unsigned int m
 		CC (entry->components, 4, v, maxlen);
 		vl = exif_get_long (entry->data, entry->order);
 		if ((vl & 0xF0F0F0F0) == 0x30303030) {
-			memcpy (v, entry->data, MIN(maxlen, 4));
+			memcpy (v, entry->data, MIN (maxlen, 4));
 		} else {
 			snprintf (v, maxlen, "%04lx", vl);
 		}
@@ -418,12 +426,13 @@ mnote_olympus_entry_get_value (MnoteOlympusEntry *entry, char *v, unsigned int m
 		break;
 	case MNOTE_OLYMPUS_TAG_VERSION:
 		CF (entry->format, EXIF_FORMAT_ASCII, v, maxlen);
-		CC (entry->components, 5, v, maxlen);
+		CC2 (entry->components, 5, 8, v, maxlen);
 		strncpy (v, entry->data, MIN (maxlen, entry->size));
 		break;
 	case MNOTE_OLYMPUS_TAG_INFO:
 		CF (entry->format, EXIF_FORMAT_ASCII, v, maxlen);
-		CC (entry->components, 52, v, maxlen);
+		CC2 (entry->components, 52, 53, v, maxlen);
+		strncpy (v, entry->data, MIN (maxlen, entry->size));
 		break;
 	case MNOTE_OLYMPUS_TAG_ID:
 		CF (entry->format, EXIF_FORMAT_UNDEFINED, v, maxlen);
@@ -515,11 +524,11 @@ mnote_olympus_entry_get_value (MnoteOlympusEntry *entry, char *v, unsigned int m
 			break;
 		case EXIF_FORMAT_UNDEFINED:
 		default:
-			snprintf (v, maxlen, "%li bytes unknown data:",
+			snprintf (v, maxlen, _("%li bytes unknown data: "),
 				  entry->size);
 			for (i = 0; i < entry->size; i++) {
-				sprintf (buf, " %02x", entry->data[i]);
-				strncat (v, buf, maxlen);
+				sprintf (buf, "%02x", entry->data[i]);
+				strncat (v, buf, maxlen - strlen (v));
 			}
 			break;
 		}
